# -*- coding: utf-8 -*-
import glob
import law
import luigi
import logging
import os
import re

from itertools import product
from subprocess import PIPE

from law.util import colored

from analysis.tasks.MergeFinal import MergeFinal

from analysis.framework import Task, SpaceSeparatedListParameter
from analysis.util import interruptable_popen

logger = logging.getLogger(__name__)


class FnloCppreadFinal(Task, law.LocalWorkflow):
    merge_dir = luigi.Parameter(default='grids')

    default_channels = ["LO", "R", "V", "RRa", "RRb", "RV", "VV", "NLO", "NLO_only", "NNLO", "NNLO_only"]
    channels = SpaceSeparatedListParameter(
        default=default_channels,
        cls=luigi.Parameter,
        choices=("LO", "R", "V", "RR", "RRa", "RRb", "RV", "VV", "NLO", "NLO_only", "NNLO", "NNLO_only")
    )
    observables = luigi.ListParameter()

    pdf = luigi.Parameter()
    scalecombs = SpaceSeparatedListParameter()
    ascode = luigi.Parameter()
    norm = luigi.Parameter()
    scale = SpaceSeparatedListParameter()

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output."""
        return {
            i : {'channel': channel, 'observable': obs}
            for i, (channel, obs) in enumerate(product(
                self.channels,
                self.observables
            ))
        }

    def workflow_requires(self):
        return {'mergefinal': MergeFinal()}

    def requires(self):
        return MergeFinal()

    def output(self):
        out_dir = os.getenv("ANALYSIS_PATH") or os.getcwd()
        return law.LocalFileTarget(
            '{}/{}/{}/Combined/Final/{}.{}.{}.log'.format(
                out_dir,
                self.merge_dir, self.name, self.process, self.branch_data['channel'],
                self.branch_data['observable']
            )
        )

    def run(self):
        try:
            self.output().parent.touch()

            logfile = self.output().path
            table = re.sub('.log', '', logfile) + '.tab.gz'
            logger.debug('Table file is: %s', table)

            iscl = 0
            if len(self.scale) * len(self.scalecombs) > 1:
                for i, observable in enumerate(self.observables):
                    if observable in table:
                        iscl = i
                        logger.debug(
                            "Table name '%s' matches observable %s with index %s",
                            table, observable, iscl
                        )

            scl = self.scale[0]
            if len(self.scale) > 1:
                scl = self.scale[iscl]

            sclcom = self.scalecombs[0]
            if len(self.scalecombs) > 1:
                sclcom = self.scalecombs[iscl]

            logger.debug(
                'Investigating central scale %s for scale combinations %s',
                scl, sclcom
            )

            command_line_args = [
                'fnlo-tk-cppread', table, self.pdf, sclcom,
                self.ascode, self.norm, scl
            ]
            logger.info("Running command: %s",
                colored(' '.join(command_line_args), 'yellow'))

            code, out, error = interruptable_popen(command_line_args,
                                                   stdout=PIPE,
                                                   stderr=PIPE)
            logger.debug("Return code: %d", code)
            logger.debug("Standard output:\n%s\n", out)
            logger.debug("Standard error:\n%s\n", error)

            if (code == 0):
                logger.info("fnlo-tk-cppread status: SUCCESS")

                # on success, write output to log file
                with open(logfile, 'w') as outfile:
                    outfile.write(out)
            else:
                raise RuntimeError("fnlo-tk-cppread status: FAIL (exit code {})".format(code))

        # on error, make log entry and reraise
        except Exception as e:
            logger.error('%s raised during job execution: %s', e.__class__.__name__, e)
            raise
