# -*- coding: utf-8 -*-
import law
import luigi
import logging
import os
import re
import shutil

from itertools import product
from subprocess import PIPE

from law.util import colored

from analysis.tasks.CopyWarmup import CopyWarmup

from analysis.framework import Task, SpaceSeparatedListParameter
from analysis.util import interruptable_popen

logger = logging.getLogger(__name__)


class PlotVegasGrids(Task, law.LocalWorkflow):
    command = 'nnlojet-plotgrid.sh'

    force = luigi.BoolParameter(default=False)

    warm_dir = luigi.Parameter(default='warmups')
    plots_dir = luigi.Parameter(default='plots')

    default_channels = ["LO", "R", "V", "RRa", "RRb", "RV", "VV"]
    channels = SpaceSeparatedListParameter(
        default=default_channels,
        cls=luigi.Parameter,
        choices=("LO", "R", "V", "RR", "RRa", "RRb", "RV", "VV")
    )

    cutoff = luigi.Parameter()
    jobnameext = luigi.Parameter()

    # gnuplot might NOT report errors properly (return code > 0)!
    # ==> scan stdout for these strings to check if an error occurred
    GNUPLOT_ERROR_STRINGS = (
        'STATS_columns',
    )

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output."""
        branchmap = {}
        for i, channel in enumerate(self.channels):
            branchmap[i] = {
                'index': i,
                'channel': channel
            }
        return branchmap

    def workflow_requires(self):
        if self.force:
            return {"copywarmup": CopyWarmup.req(self, acceptance=0)}
        else:
            return {"copywarmup": CopyWarmup.req(self)}

    def requires(self):
        return {
            'copywarmup': CopyWarmup(branch=self.branch_data['index'])
        }

    def output(self):
        out_dir = os.getenv("ANALYSIS_PATH") or os.getcwd()
        return law.LocalFileTarget(
            '{}/{}/{}/{}/{}.{}-{}.{}.{}_iterations.pdf'.format(
                out_dir,
                self.plots_dir, self.name, self.__class__.__name__,
                self.process, self.branch_data['channel'], self.jobnameext,
                self.cutoff, self.branch_data['channel']
            )
        )

    def run(self):
        # ensure output parent directory exists
        logger.debug("Creating output parent directory: %s", self.output().parent.path)
        self.output().parent.touch()

        gridpath = '{}/{}/{}'.format(self.warm_dir, 'warmup', self.name)
        gridfile = '{}.{}-{}.{}.{}_iterations.txt'.format(
            self.process, self.branch_data['channel'], self.jobnameext,
            self.cutoff, self.branch_data['channel']
        )
        logger.debug("Input VEGAS grid path: %s/%s", gridpath, gridfile)

        outpath = self.output().parent.path
        outfile = re.sub('.txt$', '.pdf', gridfile)
        logger.debug("Output plot path: %s/%s", outpath, outfile)

        job_ok = True

        try:

            command_line_args = [
                self.command, os.path.join(gridpath, gridfile)
            ]

            logger.info("Running command: %s",
                    colored(' '.join(command_line_args), 'yellow'))
            code, out, error = interruptable_popen(command_line_args,
                                                   stdout=PIPE,
                                                   stderr=PIPE)

            logger.debug("Return code: %d", code)
            logger.debug("Standard output:\n%s\n", out)
            logger.debug("Standard error:\n%s\n", error)

            # code may be 0 even if an error occurs -> scan stdout & stderr for error strings
            for err_string in self.GNUPLOT_ERROR_STRINGS:
                if err_string in out or err_string in error:
                    code = 1
                    ###error = out
                    break

            if (code == 0):
                logger.info("%s status: SUCCESS", self.command)
            elif error:
                if 'STATS_columns' in out or 'STATS_columns' in error:
                    logger.warning("Check gnuplot version; might be too old (<v5) to use STATS_columns!")
                raise RuntimeError("{} status: FAIL (standard error is not empty)".format(self.command))
            else:
                raise RuntimeError("{} status: FAIL (exit code {})".format(self.command, code))

        # mark job as failed on error, make log entry and reraise
        except Exception as e:
            job_ok = False
            logger.error('%s raised during job execution: %s', e.__class__.__name__, e)
            raise

        finally:
            output_src = os.path.join(gridpath, outfile)
            output_dst = os.path.join(outpath, outfile)
            if job_ok:
                # move outputs to target location
                logger.debug("Moving outputs from %s to %s", output_src, output_dst)
                shutil.move(output_src, output_dst)
            else:
                # clean up failure
                if os.path.isfile(output_src):
                    logger.debug("Removing failed task output at: %s", output_src)
                    os.remove(output_src)
                if self.output().exists():
                    logger.debug("Removing failed task output at: %s", self.output().path)
                    self.output().remove()
