# -*- coding: utf-8 -*-
import law
import luigi
import logging
import os
import shutil

from itertools import product
from subprocess import PIPE

from law.util import colored

from analysis.tasks.Recombine import Recombine

from analysis.framework import Task
from analysis.util import interruptable_popen

logger = logging.getLogger(__name__)


class StatUncertainty(Task, law.LocalWorkflow):
    command = 'fastnnlo_statunc.py'

    merge_dir = luigi.Parameter(default='grids')
    plots_dir = luigi.Parameter(default='plots')

    channellist = luigi.Parameter(default='all')
    orders = luigi.ListParameter()
    observables = luigi.ListParameter()

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output directory."""
        return {
            i : {'order': order}
            for i, order in enumerate(self.orders)
        }

    def run_configs(self):
        """Run configuration dicts. `run` method loops through these to populate the output directory."""
        return [
            {'observable': obs}
            for obs in self.observables
        ]

    def workflow_requires(self):
        return {'recombine': Recombine()}

    def output(self):
        out_dir = os.getenv("ANALYSIS_PATH") or os.getcwd()
        return law.LocalDirectoryTarget(
            '{}/{}/{}/{}/{}'.format(
                out_dir,
                self.plots_dir, self.name, self.__class__.__name__,
                self.branch_data['order']
            )
        )

    def run(self):
        try:
            with self.output().temporary_path() as self.temp_output_path:
                os.mkdir(self.temp_output_path)

                for i, run_config in enumerate(self.run_configs()):
                    tabfile = '{}/{}/Combined/Final/{}.{}.{}.tab.gz'.format(
                        self.merge_dir, self.name,
                        self.process,
                        self.branch_data['order'],
                        run_config['observable']
                    )
                    logger.debug('Investigating table: %s', tabfile)

                    outfile = '{}/{}.{}.{}'.format(
                        self.temp_output_path, self.process,
                        self.branch_data['order'],
                        run_config['observable']
                    )
                    logger.debug('Outfile name is: %s', outfile)

                    command_line_args = [
                        self.command,
                        '-c', self.channellist,
                        '-o', self.branch_data['order'],
                        '--title', run_config['observable'],
                        '-f', outfile,
                        tabfile
                    ]

                    logger.info("Running command [%s/%s]: %s", i+1,
                        len(self.run_configs()),
                        colored(' '.join(command_line_args), 'yellow'))
                    code, out, error = interruptable_popen(command_line_args,
                                                           stdout=PIPE,
                                                           stderr=PIPE)

                    logger.debug("Return code: %d", code)
                    logger.debug("Standard output:\n%s\n", out)
                    logger.debug("Standard error:\n%s\n", error)

                    if (code == 0):
                        logger.info("%s status: SUCCESS", self.command)
                    else:
                        raise RuntimeError("{} status: FAIL (exit code {})".format(self.command, code))

        # mark job as failed on error, make log entry and reraise
        except Exception as e:
            shutil.rmtree(self.temp_output_path)
            logger.error('%s raised during job execution: %s', e.__class__.__name__, e)
            raise
